unit MyProps;

interface

{$IFNDEF VER80}            { DELPHI 1.0     }
 {$IFNDEF VER90}           { DELPHI 2.0     }
  {$IFNDEF VER93}          { C++Builder 1.0 }
   {$IFNDEF VER100}        { DELPHI 3.0     }
    {$IFNDEF VER110}       { C++Builder 3.0 }
     {$IFNDEF VER120}      { DELPHI 4.0 }
      {$IFNDEF VER125}     { C++Builder 4.0 }
       {$IFNDEF VER130}    { DELPHI/C++Builder 5.0 }
        {$DEFINE D6_UP}
       {$ENDIF}
      {$ENDIF}
     {$ENDIF}
    {$ENDIF}
   {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

uses
  {$IFDEF D6_UP} Variants, {$ENDIF}
  Classes, SysUtils, FlexProps;

type
  TMyData = class(TPersistent)
  private
   FStrings: TStringList;
   FOnChange: TNotifyEvent;
  protected
   procedure DoChange; virtual;
   procedure StringsChange(Sender: TObject);
  public
   constructor Create;
   destructor Destroy; override;
   procedure Assign(Source: TPersistent); override;
   procedure SaveToStream(AStream: TStream);
   procedure LoadFromStream(AStream: TStream);
   property  Strings: TStringList read FStrings;
   property  OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TMyComplexProp = class(TCustomProp)
  private
   FValue: integer;
   FCaption: string;
   FData: TMyData;
   FFontProp: TFontProp;
   procedure SetCaption(const Value: string);
   procedure SetData(const Value: TMyData);
   procedure SetValue(const Value: integer);
   function  StoreCaption: Boolean;
   function  StoreData: Boolean;
   function  StoreValue: Boolean;
  protected
   procedure DataChange(Sender: TObject);
   function  GetDisplayValue: string; override;
   procedure SetPropValue(const PropName: string; Value: Variant); override;
   function  GetPropValue(const PropName: string): Variant; override;
   function  GetPropType(const PropName: string): TPropType; override;
  public
   constructor Create(AOwner: TPropList; const AName: string);
   destructor Destroy; override;
  published
   property  Value: integer read FValue write SetValue stored StoreValue;
   property  Caption: string read FCaption write SetCaption stored StoreCaption;
   property  Data: TMyData read FData write SetData stored StoreData;
   property  Font: TFontProp read FFontProp;
  end;

implementation

// TMyData ////////////////////////////////////////////////////////////////////

constructor TMyData.Create;
begin
 FStrings := TStringList.Create;
 FStrings.OnChange := StringsChange;
end;

destructor TMyData.Destroy;
begin
 inherited;
 FStrings.Free;
end;

procedure TMyData.Assign(Source: TPersistent);
begin
 if Source is TMyData then begin
  // Assigning
  FStrings.Assign(TMyData(Source).FStrings);
 end else
  inherited;
end;

procedure TMyData.LoadFromStream(AStream: TStream);
begin
 FStrings.LoadFromStream(AStream);
end;

procedure TMyData.SaveToStream(AStream: TStream);
begin
 FStrings.SaveToStream(AStream);
end;

procedure TMyData.DoChange;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TMyData.StringsChange(Sender: TObject);
begin
 DoChange;
end;

// TMyComplexProp /////////////////////////////////////////////////////////////

constructor TMyComplexProp.Create(AOwner: TPropList; const AName: string);
begin
 inherited;
 FData := TMyData.Create;
 FData.OnChange := DataChange;
 FPropType := ptComplex;
 FFontProp := TFontProp.Create(Nil, '');
 InitPublished(AOwner, FFontProp);
end;

destructor TMyComplexProp.Destroy;
begin
 inherited;
 FData.Free;
 FFontProp.Free;
end;

function TMyComplexProp.GetDisplayValue: string;
begin
 Result := '(MyComplex)';
end;

procedure TMyComplexProp.DataChange(Sender: TObject);
begin
 DoChanged;
end;

procedure TMyComplexProp.SetCaption(const Value: string);
begin
 if (Value = FCaption) or Owner.IsReadOnly(Self) then exit;
 DoBeforeChanged;
 FCaption := Value;
 DoChanged;
end;

procedure TMyComplexProp.SetData(const Value: TMyData);
begin
  FData := Value;
end;

procedure TMyComplexProp.SetValue(const Value: integer);
begin
 if (Value = FValue) or Owner.IsReadOnly(Self) then exit;
 DoBeforeChanged;
 FValue := Value;
 DoChanged;
end;

function TMyComplexProp.StoreCaption: Boolean;
begin
 Result := FCaption <> '';
end;

function TMyComplexProp.StoreData: Boolean;
begin
 Result := true;
end;

function TMyComplexProp.StoreValue: Boolean;
begin
 Result := FValue <> 0;
end;

function TMyComplexProp.GetPropType(const PropName: string): TPropType;
begin
 if CompareText(PropName, 'Data') = 0 then
  Result := ptHexData
 else
 if CompareText(PropName, 'Font') = 0 then
  Result := ptComplex
 else
  Result := inherited GetPropType(PropName);
end;

function TMyComplexProp.GetPropValue(const PropName: string): Variant;
var HexStream: TMemoryStream;
    VarArray: Pointer;
    Count: integer;
begin
 if CompareText(PropName, 'Data') = 0 then begin
  HexStream := TMemoryStream.Create;
  try
   // Write FData
   FData.SaveToStream(HexStream);
   Count := HexStream.Size;
   Result := VarArrayCreate([0, Count-1], varByte);
   VarArray := VarArrayLock(Result);
   try
    Move(HexStream.Memory^, VarArray^, Count);
   finally
    VarArrayUnlock(Result);
   end;
  finally
   HexStream.Free;
  end;
 end else
  // Default Get method
  Result := inherited GetPropValue(PropName);
end;

procedure TMyComplexProp.SetPropValue(const PropName: string; Value: Variant);
var VArray: Pointer;
    HexStream: TMemoryStream;
    Count: integer;
begin
 if CompareText(PropName, 'Data') = 0 then begin
  if (VarType(Value) and varArray) <> 0 then begin
   // Read FData
   HexStream := Nil;
   Count := VarArrayHighBound(Value, 1) + 1;
   VArray := VarArrayLock(Value);
   try
    HexStream := TMemoryStream.Create;
    HexStream.Write(VArray^, Count);
    HexStream.Position := 0;
    FData.LoadFromStream(HexStream);
   finally
    VarArrayUnlock(Value);
    HexStream.Free;
   end;
  end;
 end else
  // Default Set method
  inherited;
end;

end.
