unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FlexBase, FlexControls, FlexPath, FlexUtils, ComCtrls, StdCtrls, ExtCtrls,
  Buttons, Spin, Intersections;

type
  TfmMain = class(TForm)
    fpMain: TFlexPanel;
    sbrMain: TStatusBar;
    panBottom: TPanel;
    btFlatten: TButton;
    btEdit: TButton;
    sbtPointEdit: TSpeedButton;
    Bevel1: TBevel;
    btRotate: TButton;
    sedDegree: TSpinEdit;
    Bevel2: TBevel;
    Bevel3: TBevel;
    btNew1: TButton;
    btNew2: TButton;
    Bevel4: TBevel;
    lblPos: TLabel;
    tbrPosition: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure fpMainNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
    procedure btFlattenClick(Sender: TObject);
    procedure btEditClick(Sender: TObject);
    procedure btRotateClick(Sender: TObject);
    procedure btNew1Click(Sender: TObject);
    procedure btNew2Click(Sender: TObject);
    procedure sbtPointEditClick(Sender: TObject);
    procedure tbrPositionChange(Sender: TObject);
  private
    { Private declarations }
    FMarker: TFlexEllipse;
    FMarkerLastPos: TPoint;
    FCurve: TFlexCurve;
    FPointIndex: integer;
    FPathLength: single;
    FPathPos: single;
    FFormChanging: boolean;
    FIntersectionCurve: TFlexCurve;
    FIntersectionPointsLayer: TFlexLayer;
    FIntersection: TFlexIntersections;
    procedure BestFit;
    procedure UpdateMarker(const MousePos: TPoint);
    procedure UpdateStatusBar;
    procedure RebuildIntersection;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

procedure TfmMain.FormCreate(Sender: TObject);
var Layer: TFlexLayer;
    Curve: TFlexCurve;
begin
 // Initialize
 Randomize;
 LoadFlexCursors;
 fpMain.InDesign := true;
 // Load flex document
 fpMain.LoadFromFile(ExtractFilePath(ParamStr(0))+'testpoly.fxd');
 // Create layer for marker (non selectable)
 Layer := fpMain.Layers.New;
 Layer.Selectable := false;
 // Create marker control
 FMarker := TFlexEllipse.Create(fpMain, fpMain.ActiveScheme, Layer);
 with FMarker do begin
  Width := 15000;
  Height := 15000;
  BrushProp.Color := clRed;
  PenProp.Color := clMaroon;
 end;
 // Create layer for intersection points (non selectable)
 FIntersectionPointsLayer := fpMain.Layers.New;
 FIntersectionPointsLayer.Selectable := false;
 FIntersectionPointsLayer.BackOne;
 // Create intersection curve
 Curve := TFlexCurve.Create(fpMain, fpMain.ActiveScheme, fpMain.ActiveLayer);
 Curve.BeginUpdate;
 with Curve do
 try
   // Delete all points
   while PointCount > 0 do DeletePoint(0);
   AddPoint(Point(20000, 20000));
   AddPoint(Point(fpMain.DocHeight - 20000, fpMain.DocHeight - 20000));
   EndFigure(False);
   PenProp.Width := 3000;
   PenProp.Color := clOlive;
 finally
   EndUpdate;
 end;
 FIntersectionCurve := Curve;
 // Select default
 FCurve := TFlexCurve( fpMain.FindControl('Curve20') );
 if Assigned(FCurve) then FCurve.IsSelected := true;
 // Prepate intersection object
 FIntersection := TFlexIntersections.Create;
 FIntersection.SetControlA(FIntersectionCurve);
 FIntersection.SetControlB(FCurve);
 RebuildIntersection;
 // Fit document to window
 BestFit;
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
 FreeAndNil(FIntersection);
end;

procedure TfmMain.BestFit;
var AScale: TPoint;
begin
 with fpMain do
 try
  DocWidth := fpMain.DocWidth;
   //ScalePixels(StrToInt(ActiveScheme.UserData.Values['Width']));
  DocHeight := fpMain.DocHeight;
   //ScalePixels(StrToInt(ActiveScheme.UserData.Values['Height']));
  // Calculate scale
  AScale.X := (ClientWidth - 20)* 100 div UnscalePixels(DocWidth);
  AScale.Y := (ClientHeight - 20) * 100 div UnscalePixels(DocHeight);
  if AScale.X < AScale.Y
   then Scale := AScale.X
   else Scale := AScale.Y;
 except
 end;
end;

procedure TfmMain.RebuildIntersection;
const
  DotSize = 15000;
var
  i: integer;
  Dot: TFlexEllipse;
begin
 FIntersection.RebuildIntersectionPoints;
 // Clear all objects from intersection points layer
 for i:=fpMain.ActiveScheme.Count-1 downto 0 do
   if fpMain.ActiveScheme[i].Layer = FIntersectionPointsLayer then
     fpMain.ActiveScheme[i].Free;
 // Add new intersection points
 for i:=0 to FIntersection.PointCount-1 do begin
  Dot :=
    TFlexEllipse.Create(fpMain, fpMain.ActiveScheme, FIntersectionPointsLayer);
  with FIntersection.Points[i] do
    Dot.DocRect := Rect(X - DotSize div 2, Y - DotSize div 2,
      X + DotSize div 2, Y + DotSize div 2);
  Dot.BrushProp.Color := clYellow;
  Dot.PenProp.Color := clOlive;
 end;
end;

procedure TfmMain.UpdateMarker(const MousePos: TPoint);
var Control: TFlexControl;
    p: TPoint;
    Points: TPointArray;
    Types: TPointTypeArray;
    Nearest: TNearestPoint;
begin
 if (fpMain.SelectedCount = 1) and
    (fpMain.Selected[0].PointCount > 0) then begin
  // Save last mouse pos
  FMarkerLastPos := MousePos;
  Control := fpMain.Selected[0];
  // Get mouse position in control coordinates in var p
  p := Control.OwnerToClient(MousePos);
  // Get control points and point types
  Control.GetPointsEx(Points, Types);
  // Find nearets point on path
  PointOnPath(Points, Types, p, True, False, 0, @Nearest, Nil, @FPathPos);
  // Convert nearest point (in control coords) to window coordinates
  p := Control.ClientToOwner(Nearest.Point);
  // Covert nearest point to document coordinates
  fpMain.UnTransformPoint(p.x, p.y);
  // Set marker position
  with FMarker do begin
   Left := p.x - Width div 2;
   Top := p.y - Height div 2;
   Visible := true;
  end;
  FPointIndex := Nearest.MinIndex;
  // Update length
  FPathLength := PathLength(Points, Types);  
 end else begin
  // There is no curve control selected. Hide marker
  FMarker.Visible := false;
  FPointIndex := -1;
  FPathLength := -1.0;
  FPathPos := -1.0;
 end;
 // Update status bar
 UpdateStatusBar;
end;

procedure TfmMain.fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 UpdateMarker(Point(X, Y));
end;

procedure TfmMain.fpMainNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
begin
 if not Assigned(Application.MainForm) or not Assigned(FIntersection) or 
    not (Notify in [fnRect, fnEditPoints, fnSelect]) then exit;
 if Control = FIntersectionCurve then begin
  FIntersection.SetControlA(Control);
  RebuildIntersection;
 end else
 if Control.IsSelected then begin
  if Control = FIntersectionCurve
   then FIntersection.SetControlB(Nil)
   else FIntersection.SetControlB(Control);
  RebuildIntersection;
 end;
end;

procedure TfmMain.btFlattenClick(Sender: TObject);
begin
 if not Assigned(FCurve) then exit;
 // Convert curve bezier segments to line segments
 FCurve.FlattenPoints(1/PixelScaleFactor);
 // Now default curve edit method is not avail (we change points)
 btEdit.Enabled := false;
end;

procedure TfmMain.btEditClick(Sender: TObject);
var Selected: TSelectedArray;
    i: integer;
    PathEdit: TPathEditParams;
begin
 if not Assigned(FCurve) then exit;
 SetLength(Selected, FCurve.PointCount);
 for i:=0 to Length(Selected)-1 do Selected[i] := false;
 // To line
{ Selected[3] := true;
 Selected[7] := true;
 FCurve.EditPoints(pfToLine, Selected); }
 // To curve
{ Selected[0] := true;
 Selected[6] := true;
 FCurve.EditPoints(pfToCurve, Selected); }
 // Break
 Selected[7] := true;
 Selected[19] := true;
 FCurve.EditPoints(pfBreak, Selected);
 // Close 1
 SetLength(Selected, FCurve.PointCount);
 for i:=0 to Length(Selected)-1 do Selected[i] := false;
 Selected[0] := true;
 Selected[8] := true;
 FCurve.EditPoints(pfClose, Selected);
 // Close 2
 SetLength(Selected, FCurve.PointCount);
 for i:=0 to Length(Selected)-1 do Selected[i] := false;
 Selected[9] := true;
 Selected[21] := true;
 FCurve.EditPoints(pfClose, Selected);
 // Offset
 PathEdit.Offset.x := 20000;
 PathEdit.Offset.y := 0;
 SetLength(Selected, FCurve.PointCount);
 for i:=0 to Length(Selected)-1 do Selected[i] := i >= 9;
 FCurve.EditPoints(pfOffset, Selected, @PathEdit);
// FCurve.PenProp.Style := psClear;
 // Now default curve edit method is not avail (we change points)
 btEdit.Enabled := false;
end;

procedure TfmMain.sbtPointEditClick(Sender: TObject);
begin
 // Change mode
 if sbtPointEdit.Down
  then fpMain.ToolMode := ftmPointEdit
  else fpMain.ToolMode := ftmSelect;
end;

procedure TfmMain.btRotateClick(Sender: TObject);
var i: integer;
    Info: TTranslateInfo;
begin
 // Setup translate info
 with Info, fpMain.SelectedRange do begin
  Center.X := Left + (Right - Left) div 2;
  Center.Y := Top + (Bottom - Top) div 2;
  Rotate := sedDegree.Value;
  Mirror := false;
 end;
 // Rotate all selected controls
 with fpMain do begin
  BeginSelectionUpdate;
  try
   for i:=0 to SelectedCount-1 do Selected[i].Translate(Info);
  finally
   EndSelectionUpdate;
  end;
 end;
 UpdateMarker(FMarkerLastPos);
end;

procedure TfmMain.btNew1Click(Sender: TObject);
var Curve: TFlexCurve;
begin
 with fpMain do begin
  // Create curve control
  Curve := TFlexCurve.Create(fpMain, ActiveScheme, ActiveLayer);
  Curve.BeginUpdate;
  with Curve do
  try
   // Delete all points
   while PointCount > 0 do DeletePoint(0);
   // Create first figure (solid)
   AddPoint(Point(0, 20000));
   AddCurvePoints(Point(20000, 20000), Point(30000, 0), Point(50000, 40000));
   AddPoint(Point(60000, 20000));
   AddCurvePoints(Point(80000, 20000), Point(70000, 50000), Point(10000, 50000));
   EndFigure(True);
   // Create second figure (non-solid)
   AddCurvePoints(Point(0, 20000), Point(10000, -10000), Point(70000, -10000));
   AddPoint(Point(80000, 20000));
   EndFigure(False);
   BrushProp.Color := clMaroon;
   // Set random position
   Curve.Left := Round((DocWidth - 80000) * Random);
   Curve.Top := Round((DocHeight - 43000) * Random);
  finally
   EndUpdate;
  end;
 end;
end;

procedure TfmMain.btNew2Click(Sender: TObject);
const
  CurvePoints: array[0..11] of TPoint = (
   // First figure (solid)
   (X: 0;       Y: 20000),
   (X: 20000;   Y: 20000),  (X: 30000;   Y: 0),      (X: 50000;   Y: 40000),
   (X: 60000;   Y: 20000),
   (X: 80000;   Y: 20000),  (X: 70000;   Y: 50000),  (X: 10000;   Y: 50000),
   // Second figure (non-solid)
   (X: 0;       Y: 20000),  (X: 10000;   Y: -10000), (X: 70000;   Y: -10000),
   (X: 80000;   Y: 20000)
  );
  CurveTypes: array[0..11] of TPointType = (
   // First figure (solid)
   ptNode, ptNode, ptControl, ptControl, ptNode,
   ptEndNodeClose, ptControl, ptControl,
   // Second figure (non-solid)
   ptNode, ptControl, ptControl, ptEndNode
  );
var Curve: TFlexCurve;
    Points: TPointArray;
    Types: TPointTypeArray;
    i: integer;
begin
 // Create curve control in random position
 Curve := TFlexCurve(fpMain.CreateControl(TFlexCurve,
   Round((fpMain.DocWidth - 80000) * Random),
   Round((fpMain.DocHeight - 43000) * Random),
   0, 0));
 // Generate points and point types
 SetLength(Points, Length(CurvePoints));
 for i:=0 to Length(CurvePoints)-1 do Points[i] := CurvePoints[i];
 SetLength(Types, Length(CurveTypes));
 for i:=0 to Length(CurveTypes)-1 do Types[i] := CurveTypes[i];
 // Set points and types
 Curve.SetPointsEx(Points, Types);
 // Change color for solid figure(s)
 Curve.BrushProp.Color := clNavy;
end;

procedure TfmMain.UpdateStatusBar;
begin
 if FPointIndex >= 0
  then sbrMain.Panels[0].Text := Format('Point index: %d', [FPointIndex])
  else sbrMain.Panels[0].Text := '';
 if FPathLength >= 0
  then sbrMain.Panels[1].Text := Format('Path length: %.3f', [FPathLength / PixelScaleFactor])
  else sbrMain.Panels[1].Text := '';
 if FPathPos >= 0
  then sbrMain.Panels[2].Text := Format('Path pos: %.3f', [FPathPos / PixelScaleFactor])
  else sbrMain.Panels[2].Text := '';
 if FFormChanging then exit;
 FFormChanging := true;
 try
  if (FPathPos >= 0) and (FPathLength > 0)
   then tbrPosition.Position := Round(FPathPos * tbrPosition.Max / FPathLength)
   else tbrPosition.Position := 0;
 finally
  FFormChanging := false;
 end;
end;

procedure TfmMain.tbrPositionChange(Sender: TObject);
var MarkerPos: TPoint;
    Nearest: TNearestPoint;
    Control: TFlexControl;
    Points: TPointArray;
    Types: TPointTypeArray;
    NewPos: single;
begin
 if (FPathLength < 0) or FFormChanging then exit;
 if (fpMain.SelectedCount = 1) and
    (fpMain.Selected[0].PointCount > 0) then begin
  Control := fpMain.Selected[0];
  // Get control points and point types
  Control.GetPointsEx(Points, Types);
  NewPos := FPathLength * tbrPosition.Position / tbrPosition.Max;
  if not PathLengthPos(Points, Types, NewPos, @Nearest) then exit;
  FPathPos := NewPos;
  Control.ClientToOwner(Nearest.Point);
  // Convert nearest point (in control coords) to window coordinates
  MarkerPos := Control.ClientToOwner(Nearest.Point);
  // Covert nearest point to document coordinates
  fpMain.UnTransformPoint(MarkerPos.x, MarkerPos.y);
  // Set marker position
  with FMarker do begin
   Left := MarkerPos.x - Width div 2;
   Top := MarkerPos.y - Height div 2;
   Visible := true;
  end;
  // Update status bar
  FFormChanging := true;
  try
   UpdateStatusBar;
  finally
   FFormChanging := false;
  end;
 end;
end;

end.
