//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "FlexBase"
#pragma link "FlexControls"
#pragma link "FlexUtils"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  fpView->InDesign = true;
  // Load scheme background from resource
  TBrushProp *Brush = ((TFlexScheme *) fpView->ActiveScheme)->BrushProp;
  Brush->Method = bmBitmap;
  // Create empty bitmap in Brush
  Brush->InitBitmap();
  // Load brush bitmap
  Brush->Bitmap->LoadFromResourceName((int)HInstance, "BG");
  // Enable cached brush bitmap for speedup
  Brush->BitmapCache = true;
  // Create simple text box
  FTextBox = (TFlexText *) fpView->CreateControl(__classid(TFlexText),
   10000, 10000, 140000, 100000);
  FTextBox->PenProp->Style = psSolid;
  FTextBox->BrushProp->Color = TColor(0x0071CB01);
  FTextBox->TextProp->Text = Format("%d x %d",
   ARRAYOFCONST(( UnScalePixels(FTextBox->Width), 
                  UnScalePixels(FTextBox->Height) )) );
  FTextBox->Alignment = taCenter;
  FTextBox->RoundnessProp->Value = 40000;
  FTextBox->Layout = tlCenter;
  // Create red box for selecting area for display in imgView
  FFrameBox = (TFlexBox *) fpView->CreateControl(__classid(TFlexBox),
   -10000, -10000, fpView->DocWidth + 20000, fpView->DocHeight + 20000);
  FFrameBox->PenProp->Width = 3000;
  FFrameBox->PenProp->Color = clRed;
  FFrameBox->IsSelected = true;
  // Enable DoubleBuffered in panImage for prevents flicking
  panImage->DoubleBuffered = true;
  RepaintImage();
  CheckTools();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::CheckTools()
{
  sedScale->Enabled = chFixedScale->Checked;
  trbScale->Enabled = chFixedScale->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::SetFixedScale(int Value)
{
  if (FScaleChanging) return;
  FScaleChanging = true;
  try
  {
    sedScale->Text = IntToStr(Value);
    trbScale->Position = Value;
    RepaintImage();
  }
  __finally
  {
    FScaleChanging = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::RepaintImage()
{
  // Resize bitmap in imgView
  Graphics::TBitmap *Bmp = imgView->Picture->Bitmap;
  Bmp->Width = imgView->Width;
  Bmp->Height = imgView->Height;
  // Fill background
  Bmp->Canvas->Brush->Color = clGray;
  Bmp->Canvas->Brush->Style = bsSolid;
  Bmp->Canvas->FillRect(Rect(0, 0, Width, Height));
  // Init CanvasRect as imgView client area
  Types::TRect CanvasRect = imgView->ClientRect;
  // Contarct canvas rect (make gray border)
  InflateRect(&CanvasRect, -40, -20);
  // Draw black dotted bounds (CanvasRect) on imgView
  Bmp->Canvas->Brush->Style = bsClear;
  Bmp->Canvas->Pen->Color = clBlack;
  Bmp->Canvas->Pen->Style = psDot;
  Bmp->Canvas->Pen->Mode = pmCopy;
  Bmp->Canvas->Pen->Width = 1;
  InflateRect(&CanvasRect, 1, 1);
  Bmp->Canvas->Rectangle(CanvasRect.Left, CanvasRect.Top,
    CanvasRect.Right, CanvasRect.Bottom);
  InflateRect(&CanvasRect, -1, -1);
  // Get DocRect value from current FFrameBox position and size
  Types::TRect DocRect = FFrameBox->DocRect;
  // Stretch draw
  int FixedScale;
  if (chFixedScale->Checked) {
    FixedScale = sedScale->Text.ToIntDef(0);
  } else {
    FixedScale = 0;
  }
  PanelStretchDraw(imgView->Canvas, fpView, &CanvasRect, &DocRect, FixedScale,
    chAutoCenter->Checked);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::PanelStretchDraw(TCanvas *ACanvas, TFlexPanel *APanel,
  const TRect *CanvasRect, const TRect *DocRect, int FixedScale,
  bool AutoCenter)
/*var APaintRect: TRect;
    AOrigin: TPoint;
    AScale: integer;
    ADocRect: TRect;
    CalcScale: TPoint;
    Temp: integer;
    Ofs: TPoint;*/
{
  Types::TRect APaintRect;
  Types::TRect ADocRect;
  Types::TPoint CalcScale;
  Types::TPoint Ofs;
  Types::TPoint AOrigin;
  int AScale;
  int Temp;
  // Canvas rect is exact APaintRect
  APaintRect = *CanvasRect;
  if (APaintRect.Top > APaintRect.Bottom) {
    Temp = Top;
    APaintRect.Top = APaintRect.Bottom;
    APaintRect.Bottom = Temp;
  }
  if (APaintRect.Left > APaintRect.Right) {
    Temp = APaintRect.Left;
    APaintRect.Left = APaintRect.Right;
    APaintRect.Right = Temp;
  }
  if (FixedScale <= 0)
  {
    // Calc minimal scale for whole DocRect was visible in CanvasRect
    CalcScale.x =
      RectWidth(APaintRect) * PixelScaleFactor * 100 / RectWidth(*DocRect);
    CalcScale.y =
      RectHeight(APaintRect) * PixelScaleFactor * 100 / RectHeight(*DocRect);
    if (CalcScale.x < CalcScale.y) {
      AScale = CalcScale.x;
    } else {
      AScale = CalcScale.y;
    }
  } else {
    // Use scale from parameter
    AScale = FixedScale;
  }
  ADocRect = *DocRect;
  if (AutoCenter)
  {
    // Center DocRect
    Ofs.x = (RectWidth(*DocRect) -
      UnScaleValue(RectWidth(APaintRect), AScale)) / 2;
    Ofs.y = (RectHeight(*DocRect) -
      UnScaleValue(RectHeight(APaintRect), AScale)) / 2;
    OffsetRect(&ADocRect, Ofs.x, Ofs.y);
  }
  // Calc document origin for DocRect with given AScale
  AOrigin.x = ScaleValue(ADocRect.Left, AScale);
  AOrigin.y = ScaleValue(ADocRect.Top, AScale);
  // Render
  APanel->PaintTo(ACanvas, APaintRect, AOrigin, AScale, NULL,
   chPaintClipped->Checked, chPaintChildren->Checked, chPaintSelected->Checked,
   chPaintForExport->Checked);
  // Update status bar info
  sbrMain->Panels->Items[0]->Text =
    Format("Image scale: %d%%", ARRAYOFCONST((AScale)) );
  sbrMain->Panels->Items[1]->Text =
    Format("Red rect size: %d x %d", ARRAYOFCONST((
      UnScalePixels((*DocRect).Right - (*DocRect).Left),
      UnScalePixels((*DocRect).Bottom - (*DocRect).Top) )) );
  Temp = APaintRect.Left;
  sbrMain->Panels->Items[2]->Text =
    Format("Image size: %d x %d", ARRAYOFCONST((
      ((int)(APaintRect.Right - APaintRect.Left)),
      ((int)(APaintRect.Bottom - APaintRect.Top)) )) );
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::fpViewResize(TObject *Sender)
{
  RepaintImage();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::fpViewEndSelectionUpdate(TObject *Sender)
{
  if (ComponentState.Contains(csDestroying)) return;
  if (FTextBox)
  {
    // Update text label in box
	FTextBox->TextProp->Text =
	  Format("%d x %d", ARRAYOFCONST((
        UnScalePixels(FTextBox->Width), UnScalePixels(FTextBox->Height) )) );
  }
  RepaintImage();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::fpViewNotify(TObject *Sender,
      TFlexControl *Control, TFlexNotify Notify)
{
  if (ComponentState.Contains(csDestroying)) return;
  if ((Notify == fnSelect) && (chPaintSelected->Checked)) RepaintImage();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::chAutoCenterClick(TObject *Sender)
{
  RepaintImage();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::chFixedScaleClick(TObject *Sender)
{
  CheckTools();
  RepaintImage();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::chPaintOptionClick(TObject *Sender)
{
  RepaintImage();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::sedScaleChange(TObject *Sender)
{
 try
 {
   SetFixedScale(sedScale->Text.ToIntDef(0));
 }
 __except(true)
 {
 }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::trbScaleChange(TObject *Sender)
{
  SetFixedScale(trbScale->Position);
}
//---------------------------------------------------------------------------


