unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, FlexBase, FlexUtils, FlexControls, FlexProps, ComCtrls,
  StdCtrls, Spin;

type
  TfmMain = class(TForm)
    panImage: TPanel;
    sptMain: TSplitter;
    sbrMain: TStatusBar;
    panControls: TPanel;
    Panel1: TPanel;
    panFlexCaption: TPanel;
    Label2: TLabel;
    fpView: TFlexPanel;
    imgView: TImage;
    panImageCaption: TPanel;
    Label1: TLabel;
    gbPaintParams: TGroupBox;
    chPaintClipped: TCheckBox;
    chPaintChildren: TCheckBox;
    chPaintSelected: TCheckBox;
    chPaintForExport: TCheckBox;
    gbRenderOptions: TGroupBox;
    chFixedScale: TCheckBox;
    chAutoCenter: TCheckBox;
    sedScale: TEdit;
    trbScale: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure fpViewResize(Sender: TObject);
    procedure fpViewEndSelectionUpdate(Sender: TObject);
    procedure fpViewNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
    procedure chAutoCenterClick(Sender: TObject);
    procedure chPaintOptionClick(Sender: TObject);
    procedure chFixedScaleClick(Sender: TObject);
    procedure sedScaleChange(Sender: TObject);
    procedure trbScaleChange(Sender: TObject);
    procedure Label3Click(Sender: TObject);
    procedure sedScaleKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FTextBox: TFlexText;
    FFrameBox: TFlexBox;
    FScaleChanging: boolean;
    procedure CheckTools;
    procedure SetFixedScale(const Value: integer);
    procedure RepaintImage;
    procedure PanelStretchDraw(ACanvas: TCanvas; APanel: TFlexPanel;
      const CanvasRect, DocRect: TRect; FixedScale: integer = 0;
      AutoCenter: boolean = false);
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

{$IFDEF CONDITIONALEXPRESSIONS}
  {$IF CompilerVersion >= 26.0} // XE5
    {$LEGACYIFEND ON}
  {$IFEND}
  {$IF CompilerVersion >= 20.0} // Delphi/C++Builder 12.0 (Studio 2009)
uses Types;
  {$IFEND}
{$ENDIF}

procedure TfmMain.FormCreate(Sender: TObject);
begin
 fpView.InDesign := true;
 // Load scheme background from resource
 with (fpView.ActiveScheme as TFlexScheme).BrushProp do begin
  Method := bmBitmap;
  // Create empty bitmap in Brush
  InitBitmap;
  // Load brush bitmap
  Bitmap.LoadFromResourceName(HInstance, 'BG');
  // Enable cached brush bitmap for speedup
  BitmapCache := true;
 end;
 // Create simple text box
 FTextBox := TFlexText(fpView.CreateControl(TFlexText,
   10000, 10000, 140000, 100000));
 with FTextBox do begin
  PenProp.Style := psSolid;
  {BrushProp.Method := bmGradient;
  BrushProp.GradBeginColor := clGreen;
  BrushProp.GradEndColor := clLime;}
  BrushProp.Color := $0071CB01;
  TextProp.Text :=
    Format('%d x %d', [UnscalePixels(Width), UnscalePixels(Height)]);
  Alignment := taCenter;
  RoundnessProp.Value := 40000;
  Layout := tlCenter;
 end;
 // Create red box for selecting area for display in imgView 
 FFrameBox := TFlexBox(fpView.CreateControl(
   TFlexBox, -10000, -10000, fpView.DocWidth + 20000, fpView.DocHeight + 20000) );
 with FFrameBox do begin
  PenProp.Width := 3000;
  PenProp.Color := clRed;
  IsSelected := true;
 end;
 // Enable DoubleBuffered in panImage for prevents flicking
 panImage.DoubleBuffered := true;
 RepaintImage;
 CheckTools;
end;

procedure TfmMain.CheckTools;
begin
 sedScale.Enabled := chFixedScale.Checked;
 trbScale.Enabled := chFixedScale.Checked;
end;

procedure TfmMain.PanelStretchDraw(ACanvas: TCanvas; APanel: TFlexPanel;
  const CanvasRect, DocRect: TRect; FixedScale: integer = 0;
  AutoCenter: boolean = false);
var APaintRect: TRect;
    AOrigin: TPoint;
    AScale: integer;
    ADocRect: TRect;
    CalcScale: TPoint;
    Temp: integer;
    Ofs: TPoint;
begin
 // Canvas rect is exact APaintRect
 APaintRect := CanvasRect;
 with APaintRect do begin
  if Top > Bottom then begin Temp := Top; Top := Bottom; Bottom := Temp; end;
  if Left > Right then begin Temp := Left; Left := Right; Right := Temp; end;
 end;
 if FixedScale <= 0 then begin
  // Calc minimal scale for whole DocRect was visible in CanvasRect
  CalcScale.x :=
    RectWidth(APaintRect) * PixelScaleFactor * 100 div RectWidth(DocRect);
  CalcScale.y :=
    RectHeight(APaintRect) * PixelScaleFactor * 100 div RectHeight(DocRect);
  if CalcScale.x < CalcScale.y
   then AScale := CalcScale.x
   else AScale := CalcScale.y;
 end else
  // Use scale from parameter
  AScale := FixedScale;
 ADocRect := DocRect;
 if AutoCenter then begin
  // Center DocRect
  Ofs.X := (RectWidth(DocRect) -
    UnscaleValue(RectWidth(APaintRect), AScale)) div 2;
  Ofs.Y := (RectHeight(DocRect) -
    UnscaleValue(RectHeight(APaintRect), AScale)) div 2;
  OffsetRect(ADocRect, Ofs.X, Ofs.Y);
 end;
 // Calc document origin for DocRect with given AScale
 AOrigin.x := FlexUtils.ScaleValue(ADocRect.Left, AScale);
 AOrigin.y := FlexUtils.ScaleValue(ADocRect.Top, AScale);
 // Render
 APanel.PaintTo(ACanvas, APaintRect, AOrigin, AScale, Nil, {True, False, False,}
   chPaintClipped.Checked, chPaintChildren.Checked, chPaintSelected.Checked,
   chPaintForExport.Checked);
 // Update status bar info
 sbrMain.Panels[0].Text := Format('Image scale: %d%%', [AScale]);
 with DocRect do
  sbrMain.Panels[1].Text := Format('Red rect size: %d x %d',
    [UnscalePixels(Right - Left), UnscalePixels(Bottom - Top)]);
 with APaintRect do
  sbrMain.Panels[2].Text := Format('Image size: %d x %d',
    [Right - Left, Bottom - Top]);
end;

procedure TfmMain.RepaintImage;
var CanvasRect: TRect;
    DocRect: TRect;
    FixedScale: integer;
begin
 // Resize bitmap in imgView
 with imgView.Picture.Bitmap, Canvas do begin
  Width  := imgView.Width;
  Height := imgView.Height;
  // Fill background
  Brush.Color := clGray;
  Brush.Style := bsSolid;
  FillRect(Rect(0, 0, Width, Height));
  // Init CanvasRect as imgView client area
  CanvasRect := imgView.ClientRect;
  // Contarct canvas rect (make gray border)
  InflateRect(CanvasRect, -40, -20);
  // Draw black dotted bounds (CanvasRect) on imgView
  Brush.Style := bsClear;
  Pen.Color := clBlack;
  Pen.Style := psDot;
  Pen.Mode := pmCopy;
  Pen.Width := 1;
  InflateRect(CanvasRect, 1, 1);
  {$IFDEF VER120}
  Rectangle(CanvasRect.Left, CanvasRect.Top, CanvasRect.Right, CanvasRect.Bottom);
  {$ELSE}
  Rectangle(CanvasRect);
  {$ENDIF}
  InflateRect(CanvasRect, -1, -1);
 end;
 // Get DocRect value from current FFrameBox position and size
 DocRect := FFrameBox.DocRect;
 // Stretch draw
 if chFixedScale.Checked
  then FixedScale := StrToIntDef(sedScale.Text, 100)
  else FixedScale := 0;
 PanelStretchDraw(imgView.Canvas, fpView, CanvasRect, DocRect, FixedScale,
   chAutoCenter.Checked);
end;

procedure TfmMain.fpViewEndSelectionUpdate(Sender: TObject);
begin
 if csDestroying in ComponentState then exit;
 if Assigned(FTextBox) then with FTextBox do
  // Update text label in box
  TextProp.Text :=
    Format('%d x %d', [UnscalePixels(Width), UnscalePixels(Height)]);
 RepaintImage;
end;

procedure TfmMain.fpViewNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
begin
 if csDestroying in ComponentState then exit;
 if (Notify = fnSelect) and chPaintSelected.Checked then RepaintImage;
end;

procedure TfmMain.fpViewResize(Sender: TObject);
begin
 RepaintImage;
end;

procedure TfmMain.chAutoCenterClick(Sender: TObject);
begin
 RepaintImage;
end;

procedure TfmMain.chPaintOptionClick(Sender: TObject);
begin
 RepaintImage;
end;

procedure TfmMain.chFixedScaleClick(Sender: TObject);
begin
 CheckTools;
 RepaintImage;
end;

procedure TfmMain.sedScaleChange(Sender: TObject);
begin
 try
  SetFixedScale(StrToInt(sedScale.Text));
 except
 end;
end;

procedure TfmMain.trbScaleChange(Sender: TObject);
begin
 SetFixedScale(trbScale.Position);
end;

procedure TfmMain.SetFixedScale(const Value: integer);
begin
 if FScaleChanging then exit;
 FScaleChanging := true;
 try
  sedScale.Text := IntToStr(Value);
  trbScale.Position := Value;
  RepaintImage;
 finally
  FScaleChanging := false;
 end;
end;

procedure TfmMain.Label3Click(Sender: TObject);
begin
 chAutoCenter.Checked := not chAutoCenter.Checked;
end;

procedure TfmMain.sedScaleKeyPress(Sender: TObject; var Key: Char);
begin
 if ((Key < '0') or (Key > '9')) and (Key <> #8) then begin
  Key := #0;
  Beep;
 end;
end;

end.
