unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FlexBase, FlexUtils, FlexControls, FlexPath, ComCtrls, ToolWin, ImgList;

type
  TfmMain = class(TForm)
    imgTools: TImageList;
    tbrMain: TToolBar;
    tbtPick: TToolButton;
    tbtZoom: TToolButton;
    tbtPan: TToolButton;
    fpMain: TFlexPanel;
    tbtFreeHand: TToolButton;
    tbtFreeHandCurve: TToolButton;
    tbtShape: TToolButton;
    procedure tbtPickClick(Sender: TObject);
    procedure tbtZoomClick(Sender: TObject);
    procedure tbtPanClick(Sender: TObject);
    procedure tbtFreeHandClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure fpMainUpdateCursor(Sender: TObject; var ACursor: TCursor);
    procedure fpMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure fpMainPaintOver(ACanvas: TCanvas; AControl: TFlexControl;
      ChildrenOnly, SelectedOnly: Boolean);
    procedure fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure fpMainMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure tbtFreeHandCurveClick(Sender: TObject);
    procedure tbtShapeClick(Sender: TObject);
  private
    { Private declarations }
    FSurface: TBitmap;
    FSurfaceOrigin: TPoint;
    FSurfaceBounds: TRect;
    FSurfaceCurve: TFlexCurve;
    FFreeHandToCurve: boolean;
    FFreeHandLastPoint: TPoint;
    FFreeHandPainting: boolean;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

var
  FreeHandCursor: integer = 0;

procedure TfmMain.FormCreate(Sender: TObject);
begin
 LoadFlexCursors;
 fpMain.InDesign := true;
 FreeHandCursor := crLastFlexCursor +1;
 Screen.Cursors[FreeHandCursor] := LoadCursor(HInstance, 'FREEHAND_CURSOR');
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
 FSurface.Free;
end;

procedure TfmMain.tbtPickClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmSelect;
end;

procedure TfmMain.tbtZoomClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmZoom;
end;

procedure TfmMain.tbtPanClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmPan;
end;

procedure TfmMain.tbtFreeHandClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmSelect; // To reset point edit selected curve
 fpMain.ToolMode := ftmUser;
 FFreeHandToCurve := false;
end;

procedure TfmMain.tbtFreeHandCurveClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmSelect; // To reset point edit selected curve
 fpMain.ToolMode := ftmUser;
 FFreeHandToCurve := true;
end;

procedure TfmMain.tbtShapeClick(Sender: TObject);
begin
 fpMain.CreatingControlClass := Nil;
 fpMain.ToolMode := ftmPointEdit;
end;

procedure TfmMain.fpMainUpdateCursor(Sender: TObject; var ACursor: TCursor);
begin
 if fpMain.ToolMode = ftmUser then ACursor := FreeHandCursor;
end;

procedure TfmMain.fpMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var PixelPos: TPoint;
    //Org: TPoint;
    BoundsInited: boolean;
begin
 if fpMain.ToolMode <> ftmUser then exit;
 fpMain.UnselectAll;
 if FFreeHandToCurve then begin
  FSurfaceCurve := TFlexCurve.Create(fpMain, fpMain.ActiveScheme,
    fpMain.ActiveLayer);
  // Convert to document coords
  PixelPos.X := X;
  PixelPos.Y := Y;
  PixelPos := fpMain.Schemes.OwnerToClient(PixelPos);
  // Set position
  FSurfaceCurve.Left := PixelPos.X;
  FSurfaceCurve.Top := PixelPos.Y;
  // Curve created with two points (initialy)
  // Delete second point
  if FSurfaceCurve.PointCount > 1 then FSurfaceCurve.DeletePoint(1);
 end else begin
  // Create surface
  if not Assigned(FSurface) then begin
   FSurface := TBitmap.Create;
   FSurface.Width := Screen.Width;
   FSurface.Height := Screen.Height;
  end;
  // Clear surface
  with FSurface.Canvas do begin
   Brush.Color := clFuchsia;
   Brush.Style := bsSolid;
   FillRect(Rect(0, 0, FSurface.Width, FSurface.Height));
  end;
  FSurface.TransparentColor := clFuchsia;
  FSurface.Transparent := true;
  // Set surface offset (origin)
  FSurfaceOrigin.X := 0;
  FSurfaceOrigin.Y := 0;
  fpMain.ClientToScreen(FSurfaceOrigin);
  // Copy selected image into surface if needed
  BoundsInited := false;
 { if (fpMain.SelectedCount = 1) and
     (fpMain.Selected[0] is TFlexPicture) and
     TFlexPicture(fpMain.Selected[0]).PictureProp.IsLoaded then begin
   FSurfaceBounds := fpMain.Selected[0].PaintRect;
   Org.X := FSurfaceBounds.Left + fpMain.Origin.X;
   Org.Y := FSurfaceBounds.Top + fpMain.Origin.Y;
   FSurface.Canvas.Draw(Org.X, Org.Y,
     TFlexPicture(fpMain.Selected[0]).PictureProp.Graphic);
   OffsetRect(FSurfaceBounds, FSurfaceOrigin.X, FSurfaceOrigin.Y);
   BoundsInited := true;
  end; } // :)))
  // Set point
  PixelPos.X := FSurfaceOrigin.X + X;
  PixelPos.Y := FSurfaceOrigin.X + Y;
  FSurface.Canvas.Pixels[PixelPos.X, PixelPos.Y] := clBlack;
  FFreeHandLastPoint := PixelPos;
  // Initialize bounds
  if not BoundsInited then begin
   FSurfaceBounds.TopLeft := PixelPos;
   FSurfaceBounds.BottomRight := PixelPos;
  end;
  // Refresh flex panel
  fpMain.Invalidate;
 end;
 // Paint begins
 FFreeHandPainting := true;
end;

procedure TfmMain.fpMainMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var PixelPos: TPoint;
    LastNode: integer;
    LastNodeType: TPointType;
    Index: integer;
begin
 if not FFreeHandPainting then exit;
 if FFreeHandToCurve then begin
  FSurfaceCurve.BeginUpdate;
  try
   // Change last node type
   LastNode := FSurfaceCurve.NodeCount-1;
   LastNodeType := FSurfaceCurve.NodeTypes[LastNode];
   FSurfaceCurve.NodeTypes[LastNode] := ptNode;
   // Convert to curve coordinates
   PixelPos.X := X;
   PixelPos.Y := Y;
   PixelPos := FSurfaceCurve.OwnerToClient(PixelPos);
   // Add new node
   Index := FSurfaceCurve.AddPoint(PixelPos);
   // Change type
   FSurfaceCurve.PointTypes[Index] := LastNodeType;   
  finally
   FSurfaceCurve.EndUpdate;
  end;
 end else
 if Assigned(FSurface) then begin
  // Set point
  PixelPos.X := FSurfaceOrigin.X + X;
  PixelPos.Y := FSurfaceOrigin.X + Y;
  if (PixelPos.X = FFreeHandLastPoint.X) or
     (PixelPos.Y = FFreeHandLastPoint.Y) then exit;
  FSurface.Canvas.MoveTo(FFreeHandLastPoint.X, FFreeHandLastPoint.Y);
  FSurface.Canvas.LineTo(PixelPos.X, PixelPos.Y);
  FFreeHandLastPoint := PixelPos;
  // Update bounds
  with FSurfaceBounds, PixelPos do begin
   if X < Left then Left := X else
   if X > Right then Right := X;
   if Y < Top then Top := Y else
   if Y > Bottom then Bottom := Y;
  end;
  // Refresh flex panel
  fpMain.Invalidate;
 end;
end;

procedure TfmMain.fpMainMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var Picture: TFlexPicture;
    Bmp: TBitmap;
    PicBounds: TRect;
begin
 if not FFreeHandPainting then exit;
 try
  if FFreeHandToCurve then begin
   // Select created curve
   FSurfaceCurve.IsSelected := true;
   FSurfaceCurve := Nil;
  end else
  if Assigned(FSurface) then begin
   // Create picture control
   Picture := TFlexPicture.Create(fpMain, fpMain.ActiveScheme,
     fpMain.ActiveLayer);
   // Convert FSurfaceBounds to document coordinates PicBounds
   PicBounds := FSurfaceBounds;
   OffsetRect(PicBounds, -FSurfaceOrigin.X, -FSurfaceOrigin.Y);
   PicBounds.TopLeft :=
     fpMain.ActiveScheme.OwnerToClient(PicBounds.TopLeft);
   PicBounds.BottomRight :=
     fpMain.ActiveScheme.OwnerToClient(PicBounds.BottomRight);
   // Set picture position 
   Picture.Left := PicBounds.Left;
   Picture.Top := PicBounds.Top;
   // Set bitmap
   Bmp := TBitmap.Create;
   with FSurfaceBounds do
   try
    Bmp.Width := Right - Left +1;
    Bmp.Height := Bottom - Top +1;
    BitBlt(Bmp.Canvas.Handle, 0, 0, Bmp.Width, Bmp.Height,
      FSurface.Canvas.Handle, Left, Top, SRCCOPY);
    Picture.PictureProp.Graphic := Bmp;
    // Set size
    Picture.Width := Bmp.Width * PixelScaleFactor;
    Picture.Height := Bmp.Height * PixelScaleFactor;
    // Set transparent
    Picture.PictureProp.MaskColor := FSurface.TransparentColor;
    Picture.PictureProp.Masked := true;
    // Select picture
    Picture.IsSelected := true;
   finally
    Bmp.Free;
   end;
  end;
 finally
  FFreeHandPainting := false;
 end;
end;

procedure TfmMain.fpMainPaintOver(ACanvas: TCanvas; AControl: TFlexControl;
  ChildrenOnly, SelectedOnly: Boolean);
begin
 if not Assigned(FSurface) or not FFreeHandPainting or FFreeHandToCurve or
   ChildrenOnly or SelectedOnly then exit;
 ACanvas.Draw(-FSurfaceOrigin.X, -FSurfaceOrigin.Y, FSurface);
end;

end.
