unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FlexBase, FlexSatellite, FlexUtils, FlexControls, FlexProps, FlexLibs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TfmMain = class(TForm)
    fpView: TFlexPanel;
    panBottom: TPanel;
    gbAnimation: TGroupBox;
    chAnimate: TCheckBox;
    lblMoon: TLabel;
    lblEarth: TLabel;
    tbrMoon: TTrackBar;
    tbrEarth: TTrackBar;
    tmAnimate: TTimer;
    rgbMoon: TRadioGroup;
    chShowOrbits: TCheckBox;
    chFixRadius: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure tmAnimateTimer(Sender: TObject);
    procedure chAnimateClick(Sender: TObject);
    procedure fpViewNotify(Sender: TObject; Control: TFlexControl;
      Notify: TFlexNotify);
    procedure rgbMoonClick(Sender: TObject);
    procedure chShowOrbitsClick(Sender: TObject);
    procedure chFixRadiusClick(Sender: TObject);
    procedure fpViewDblClick(Sender: TObject);
  private
    { Private declarations }
    FSun: TFlexSatellite;
    FEarth: TFlexSatellite;
    FMoon: TFlexSatellite;
    procedure CreateSystem;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

{ TfmMain }

procedure TfmMain.FormCreate(Sender: TObject);
var ZoomRect: TRect;
begin
 LoadFlexCursors;
 fpView.InDesign := true;
 CreateSystem;
 rgbMoon.ItemIndex := 0;
 // Set sun to center of visible area
 ZoomRect := FSun.DocRect;
 fpView.Zoom(fpView.Scale, @ZoomRect);
end;

procedure TfmMain.CreateSystem;
var AHint: TFlexText;
    HintLayer: TFlexLayer;
begin
 // Create Hint
 HintLayer := fpView.Layers.New;
 HintLayer.ToBack;
 HintLayer.Selectable := false;
 AHint := TFlexText.Create(fpView, fpView.ActiveScheme, HintLayer);
 AHint.Left := 10000;
 AHint.Top := 10000;
 AHint.FontProp.Color := clGray;
 AHint.TextProp.Text :=
   'You can select any control(s)  '#13#10 +
   '(Sun, Earth, Moon or their combinations)   '#13#10 +
   'and move/size it in any time  ';
 AHint.AutoSizeProp.Value := true;
 // Create Sun
 FSun := TFlexSatellite.Create(fpView, fpView.ActiveScheme,
   fpView.ActiveLayer);
 FSun.Width := 100000;
 FSun.Height := 100000;
 FSun.Left := 250000;
 FSun.Top := 250000;
 FSun.BrushProp.Color := clYellow;
 FSun.PenProp.Color := clRed;
 FSun.Name := 'Sun';
 FSun.Hint := FSun.Name;
 // Create Earth
 FEarth := TFlexSatellite.Create(fpView, fpView.ActiveScheme,
   fpView.ActiveLayer);
 FEarth.Width := 60000;
 FEarth.Height := 60000;
 FEarth.BrushProp.Color := clLime;
 FEarth.PenProp.Color := clGreen;
 FEarth.LinkProp.LinkedControl := FSun;
 FEarth.ShowOrbitProp.Value := true;
 FEarth.LinkRadiusProp.Value := 190000;
 FEarth.Name := 'Earth';
 FEarth.Hint := FEarth.Name;
 // Create Moon
 FMoon := TFlexSatellite.Create(fpView, fpView.ActiveScheme,
   fpView.ActiveLayer);
 FMoon.Width := 40000;
 FMoon.Height := 40000;
 FMoon.BrushProp.Color := clSilver;
 FMoon.PenProp.Color := clGray;
 FMoon.LinkProp.LinkedControl := FEarth;
 FMoon.ShowOrbitProp.Value := true;
 FMoon.LinkRadiusProp.Value := 80000;
 FMoon.Name := 'Moon';
 FMoon.Hint := FMoon.Name;
end;

procedure TfmMain.tmAnimateTimer(Sender: TObject);
begin
 with FEarth.LinkAngleProp do Value := Value + tbrEarth.Position;
 with FMoon.LinkAngleProp do Value := Value + tbrMoon.Position;
end;

procedure TfmMain.chAnimateClick(Sender: TObject);
begin
 tmAnimate.Enabled := chAnimate.Checked;
end;

procedure TfmMain.fpViewNotify(Sender: TObject; Control: TFlexControl;
  Notify: TFlexNotify);
begin
 // Orbit can't be selected
 if (Notify = fnSelect) and (Control is TFlexOrbit) then
  Control.IsSelected := false;
end;

procedure TfmMain.rgbMoonClick(Sender: TObject);
begin
 if rgbMoon.ItemIndex = 0
  then FMoon.LinkProp.LinkedControl := FEarth
  else FMoon.LinkProp.LinkedControl := FSun;
end;

procedure TfmMain.chShowOrbitsClick(Sender: TObject);
var i: integer;
    Satellites: TList;
begin
 Satellites := TList.Create;
 try
  // Collect all satellite controls before "show orbit" changing
  // since it can destroys orbit controls and change ActiveScheme.Count
  for i:=0 to fpView.ActiveScheme.Count-1 do
   if fpView.ActiveScheme[i] is TFlexSatellite then
    Satellites.Add(fpView.ActiveScheme[i]);
  // Apply show orbits
  for i:=0 to Satellites.Count-1 do
   TFlexSatellite(Satellites[i]).ShowOrbitProp.Value := chShowOrbits.Checked;
 finally
  Satellites.Free;
 end;
end;

procedure TfmMain.chFixRadiusClick(Sender: TObject);
var i: integer;
    Control: TFlexControl;
begin
 for i:=0 to fpView.ActiveScheme.Count-1 do begin
  Control := fpView.ActiveScheme[i];
  if Control is TFlexSatellite then
   TFlexSatellite(Control).LinkRadiusFixedProp.Value := chFixRadius.Checked;
 end;
end;

procedure TfmMain.fpViewDblClick(Sender: TObject);
var i: integer;
    Notify: TNotifyLink;
begin
 if (fpView.SelectedCount = 1) and
    (fpView.Selected[0] is TFlexSatellite) then begin
  // Select all linked controls
  Notify := fpView.Selected[0].NotifyLink;
  for i:=0 to Notify.LinkCount-1 do
   if Notify.Links[i].Owner is TLinkProp then
    TFlexControl(
      TCustomProp(Notify.Links[i].Owner).Owner.Owner).IsSelected := true;
 end;
end;

end.
