//---------------------------------------------------------------------------

#pragma hdrstop

#include "FlexMultiPics.h"
#include "fMultiPic.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

void __fastcall VariantToPictureList(TList *PicList, Variant &Value)
{
  if (VarIsEmpty(Value)
     #if (__BORLANDC__ > 0x0550)
     || VarIsClear(Value)
     #endif
     || (!VarIsArray(Value)))
    return;
  int ASize = VarArrayHighBound(Value, 1) +1;
  if (ASize < 4)
    return;
  TMemoryStream *MS = new TMemoryStream;
  try
  {
    void *Array = VarArrayLock(Value);
    try
    {
      MS->Write(Array, ASize);
    }
    __finally
    {
      VarArrayUnlock(Value);
    };
    MS->Position = 0;
    int ACount;
    MS->Read(&ACount, sizeof(ACount));
    for (int i = 0; i < ACount; i++)
    {
      TPicture *Picture = new TPicture;
      try
      {
        TStreamProc ReadStream;
        TStreamProc WriteStream;
        Flexutils::GetPicReadWrite(Picture, ReadStream, WriteStream);
        ReadStream(MS);
        PicList->Add(Picture);
      }
      catch(...)
      {
        Picture->Free();
        throw;
      }
    }
  }
  __finally
  {
    MS->Free();
  }
}

void __fastcall PictureListToVariant(TList *PicList, Variant &Value)
{
  TMemoryStream *MS = new TMemoryStream;
  try
  {
    int ACount = PicList->Count;

    MS->Write(&ACount, sizeof(ACount));
    for (int i=0; i < ACount; i++)
    {
      TStreamProc ReadStream;
      TStreamProc WriteStream;
      Flexutils::GetPicReadWrite((TPicture *)PicList->Items[i],
        ReadStream, WriteStream);
      WriteStream(MS);
    }

    int Bounds[2] = {0, MS->Size};
    Value = VarArrayCreate(Bounds, 1, System::varByte);
    void * Array = VarArrayLock(Value);
    try
    {
      Move(MS->Memory, Array, MS->Size);
    }
    __finally
    {
      VarArrayUnlock(Value);
    };
  }
  __finally
  {
    MS->Free();
  }
}

// TFlexMultiPicControl ///////////////////////////////////////////////////////

void _fastcall TFlexMultiPicControl::ControlCreate(void)
{
  inherited::ControlCreate();
  FPictureList = new TList;
}

void __fastcall TFlexMultiPicControl::ControlDestroy(void)
{
  inherited::ControlDestroy();
  ClearPictureList();
  FPictureList->Free();
}

void __fastcall TFlexMultiPicControl::CreateProperties(void)
{
  inherited::CreateProperties();
  FPictureIndexProp = new TIntProp(Props, "PictureIndex");
  FPictureListProp = new TDataProp(Props, "PictureList");
  FPictureListProp->OnGetPropData = GetPictureListData;
  FPictureListProp->OnSetPropData = SetPictureListData;
  FPictureListProp->Style << (psVisible);
  FPictureListProp->EditFormClass = __classid(TfmMultiPicEdit);
  FPictureListProp->Style << (psEditForm);
}

void __fastcall TFlexMultiPicControl::ClearPictureList(void)
{
  for (int i=0; i < FPictureList->Count; i++)
    ((TPicture *)FPictureList->Items[i])->Free();
  FPictureList->Clear();
}

void __fastcall TFlexMultiPicControl::PropChanged(System::TObject* Sender,
  Flexprops::TCustomProp* Prop)
{
  if ((FPictureList) && (FPictureIndexProp->Value >= 0) &&
      (FPictureIndexProp->Value < FPictureList->Count))
  {
    if (Prop == FPictureIndexProp)
    {
      PictureProp->Graphic =
        ((TPicture *) FPictureList->Items[FPictureIndexProp->Value])->Graphic;
    }
    if (Prop == FPictureListProp)
      // Direct picture list changing
      PictureListItemChange(
        (TObject *) FPictureList->Items[FPictureIndexProp->Value]);
  }
  inherited::PropChanged(Sender, Prop);
}

void __fastcall TFlexMultiPicControl::PropStored(System::TObject* Sender,
  Flexprops::TCustomProp* Prop, bool &IsStored, const UnicodeString PropName)
{
  if ((Prop == PictureProp) && (FPictureList->Count > 0))
    IsStored = FALSE;
  else
  if (Prop == FPictureIndexProp)
    IsStored = (FPictureIndexProp->Value > 0);
  else
  if (Prop == FPictureListProp)
    IsStored = (FPictureList->Count > 0);
  else
    inherited::PropStored(Sender, Prop, IsStored, PropName);
}

int __fastcall TFlexMultiPicControl::AddImage(TPicture *ASource)
{
  TPicture *Picture = new TPicture();
  if (ASource)
    Picture->Assign(ASource);
  int Index = FPictureList->Add(Picture);
  Picture->OnChange = PictureListItemChange;
  if ((FPictureIndexProp->Value >= Index) &&
      (FPictureIndexProp->Value < FPictureList->Count))
    PictureListItemChange(Images[FPictureIndexProp->Value]);
  return Index;
}

void __fastcall TFlexMultiPicControl::DeleteImage(int Index)
{
  TPicture *Picture = (TPicture *) FPictureList->Items[Index];
  Picture->OnChange = NULL;
  FPictureList->Delete(Index);
  Picture->Free();
  if (FPictureIndexProp->Value >= FPictureList->Count)
    FPictureIndexProp->Value = FPictureList->Count - 1;
  else
  if (FPictureIndexProp->Value >= Index)
    PictureListItemChange(Images[FPictureIndexProp->Value]);
}

int __fastcall TFlexMultiPicControl::GetImageCount(void)
{
  return FPictureList->Count;
}

TPicture * __fastcall TFlexMultiPicControl::GetImage(int Index)
{
  return (TPicture *) FPictureList->Items[Index];
}

void __fastcall TFlexMultiPicControl::PictureListItemChange(
  System::TObject* Sender)
{
  int Index = FPictureList->IndexOf(Sender);
  if (Index == FPictureIndexProp->Value)
    // Reload current picture
    PictureProp->Graphic = ((TPicture *) Sender)->Graphic;
}

void __fastcall TFlexMultiPicControl::SetPictureListData(
  System::TObject* Sender, Variant &Value)
{
  ClearPictureList();
  VariantToPictureList(FPictureList, Value);
  for (int i = 0; i < FPictureList->Count; i++)
    ((TPicture *) FPictureList->Items[i])->OnChange = PictureListItemChange;
  // Refresh current image
  if ((FPictureIndexProp->Value >= 0) &&
      (FPictureIndexProp->Value < FPictureList->Count))
    PictureListItemChange((TObject *)
      FPictureList->Items[FPictureIndexProp->Value]);
}

void __fastcall TFlexMultiPicControl::GetPictureListData(
  System::TObject* Sender, Variant &Value)
{
  PictureListToVariant(FPictureList, Value);
}

void RegisterFlexMultiPicControl()
{
  RegisterFlexControl(__classid(TFlexMultiPicControl));
}
