//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "fMultiPic.h"
#include "FlexMultiPics.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfmMultiPicEdit *fmMultiPicEdit;
//---------------------------------------------------------------------------
__fastcall TfmMultiPicEdit::TfmMultiPicEdit(TComponent* Owner)
        : TForm(Owner)
{
  FPictureList = new TList();
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::FormDestroy(TObject *Sender)
{
  if (ModalResult == mrOk)
    SaveToData();
    
  ClearPictureList();
  FPictureList->Free();
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::FormShow(TObject *Sender)
{
  if ((Tag != 0) && (((TObject *) Tag)->InheritsFrom(__classid(TDataProp))))
  {
    FDataProp = (TDataProp *) Tag;
    // Try to find picture index property
    TObject *Control = FDataProp->Owner->Owner;
    if (((TObject *) Control)->InheritsFrom(__classid(TFlexControl)))
      FIndexProp =
        (TIntProp *) ((TFlexControl *) Control)->Props->Props["PictureIndex"];
  }

  LoadFromData();
  BuildImageList();

  // Set current image
  if ((FIndexProp) && (FIndexProp->Value < FPictureList->Count))
    PictureIndex = FIndexProp->Value;
}
//---------------------------------------------------------------------------
int __fastcall TfmMultiPicEdit::GetPictureIndex(void)
{
  #if (__BORLANDC__ > 0x0550)
    return lvPictures->ItemIndex;
  #else
    TListItem *Item = lvPictures->ItemFocused;
    return lvPictures->Items->IndexOf(Item);
  #endif
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::SetPictureIndex(int Value)
{
  #if (__BORLANDC__ > 0x0550)
    lvPictures->ItemIndex = Value;
  #else
    lvPictures->ItemFocused = lvPictures->Items->Item[Value];
  #endif
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::ClearPictureList(void)
{
  for (int i=0; i < FPictureList->Count; i++)
    ((TPicture *)FPictureList->Items[i])->Free();
  FPictureList->Clear();
}
//---------------------------------------------------------------------------
class TDataPropFriend : public TDataProp { };

void __fastcall TfmMultiPicEdit::LoadFromData(void)
{
  if (FDataProp)
  {
    Variant Data = ((TDataPropFriend *)FDataProp)->PropData[""];
    VariantToPictureList(FPictureList, Data);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::SaveToData(void)
{
  if (FDataProp)
  {
    Variant Data;
    PictureListToVariant(FPictureList, Data);

    TObject *Control = FDataProp->Owner->Owner;
    if (Control->InheritsFrom(__classid(TFlexMultiPicControl)))
      ((TFlexMultiPicControl *) Control)->SetPictureListData(FDataProp, Data);
    else
      ((TDataPropFriend *)FDataProp)->PropData[""] = Data;
  }
  if (FIndexProp)
    FIndexProp->Value = PictureIndex;
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::BuildImageList(void)
{

  Graphics::TBitmap *bmp = NULL;
  lvPictures->Items->BeginUpdate();
  try
  {
    lvPictures->Items->Clear();
    imgIcons->Clear();
    bmp = new Graphics::TBitmap;
    bmp->Width = imgIcons->Width;
    bmp->Height = imgIcons->Height;
    // Load images to list view
    for (int i = 0; i < FPictureList->Count; i++)
    {
      bmp->Canvas->Brush->Color = clWindow;
      bmp->Canvas->FillRect(TRect(0, 0, bmp->Width, bmp->Height));
      bmp->Canvas->StretchDraw(TRect(0, 0, bmp->Width, bmp->Height),
        ((TPicture *)FPictureList->Items[i])->Graphic);
      imgIcons->Add(bmp, NULL);
      TListItem *Item = lvPictures->Items->Add();
      Item->Caption = IntToStr(i);
      Item->ImageIndex = i;
    }
  }
  __finally
  {
    lvPictures->Items->EndUpdate();
    bmp->Free();
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::CheckTools(void)
{
  bbDelete->Enabled = (PictureIndex >= 0);
}
//---------------------------------------------------------------------------
void __fastcall TfmMultiPicEdit::bbAddClick(TObject *Sender)
{
  if (!opd_Picture->Execute())
    return;

  int NewIndex;
  TPicture *Picture = new TPicture;
  try
  {
    Picture->LoadFromFile(opd_Picture->FileName);
    NewIndex = FPictureList->Add(Picture);
  }
  __except(EXCEPTION_EXECUTE_HANDLER)
  {
    Picture->Free();
    throw;
  }
  BuildImageList();
  PictureIndex = NewIndex;
  CheckTools();
}
//---------------------------------------------------------------------------

void __fastcall TfmMultiPicEdit::bbDeleteClick(TObject *Sender)
{
  if (PictureIndex < 0)
    return;

  int Index = PictureIndex;
  TPicture * Picture = (TPicture *) FPictureList->Items[Index];
  FPictureList->Delete(Index);
  Picture->Free();

  BuildImageList();
  if (Index < lvPictures->Items->Count)
    PictureIndex = Index;
  else
    PictureIndex = lvPictures->Items->Count -1;
    
  CheckTools();
}
//---------------------------------------------------------------------------

